/*
		View for TI89/TI92+
		Copyright (C) 2000  Matus Horvath (m_horvath@geocities.com)

		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

		-------
		Latest version of this program (in source and binary form) can be found
		at http://www.elf.stuba.sk/~horvathm. The GNU General Public Licence can
		be found at http://www.gnu.org.
*/

#ifndef _MACROS_H
#define _MACROS_H

#include <compat.h>

// BUILDVER returns version string
#if (defined DEBUG)&&(defined FINAL)
#error You cannot make FINAL DEBUG version ...
#elif (defined DEBUG)&&(!defined FINAL)
#define BUILDVER VER_HI"."VER_ME"."VER_LO""VER_SPC" DBG build "VER_BLD
#elif (!defined DEBUG)&&(!defined FINAL)
#define BUILDVER VER_HI"."VER_ME"."VER_LO""VER_SPC" testing build "VER_BLD
#elif (!defined DEBUG)&&(defined FINAL)
#define BUILDVER VER_HI"."VER_ME"."VER_LO""VER_SPC
#endif

// X and Y position of error messages
#define ERRMSGX 0
#define ERRMSGY 10

// Line height in pixels
#define LINEHEIGHT 7

// Tabelator size in pixels
#define TABSIZE 20

// Wrap lines on word boundaries (1=yes)
static int WORDWRAP=1;

// Proportional font (1 = yes)
static int PROPFONT=1;
// Maximum character width (m)
#define MAXCHARWIDTH 6

// Used for "paging", !do not alter this!
#define BL1SIZE 32
#define BL2SIZE 32
#define BL3SIZE 64

#define EXIT goto exit;	// TODO: get rid of this
#define _END_ exit:

// Displays error message
void ERROR(char *msg) {
	DrawStr(ERRMSGX,ERRMSGY,"Error: ",A_REPLACE); \
	DrawStr(ERRMSGX+30,ERRMSGY,msg,A_REPLACE); \
	ngetchx(); \
}

// Used for debugging only
#ifdef DEBUG
char DISPBUF[256]="";
#define DISP(type,var) do { \
	sprintf(DISPBUF,#var"="type"      ",var); \
	DrawStr(0,0,DISPBUF,A_REPLACE); \
	ngetchx(); \
} while(0)

#define DISP2(type,var1,var2) do { \
	sprintf(DISPBUF,#var1"="type", "#var2"="type"      ",var1,var2); \
	DrawStr(0,0,DISPBUF,A_REPLACE); \
	ngetchx(); \
} while(0)

#define DISPNW(type,var) do { \
	sprintf(DISPBUF,#var"="type"      ",var); \
	DrawStr(0,0,DISPBUF,A_REPLACE); \
} while(0)

#define DISPS(var) DISP("%s",var);
#define DISPC(var) DISP("%c",var);
#define DISPI(var) DISP("%i",var);
#define DISPSNW(var) DISPNW("%s",var);
#define DISPINW(var) DISPNW("%i",var);
#else
#define DISP(type,var) ;
#define DISPS(var) ;
#define DISPC(var) ;
#define DISPI(var) ;
#define DISPNW(type,var) ;
#define DISPSNW(var) ;
#define DISPINW(var) ;
#define DISP2(type,var1,var2) ;
#endif


#ifdef NOSTUB
#define __declare_global_int(declarator) declarator=0
#else
#define __declare_global_int(declarator) declarator
#endif

__declare_global_int(unsigned int MAXX);
__declare_global_int(unsigned int MAXY);
__declare_global_int(unsigned int MAXLINE);
__declare_global_int(unsigned int OPENMENULEN);
__declare_global_int(unsigned int ABOUTDLGH);
__declare_global_int(unsigned int ABOUTDLGW);
__declare_global_int(unsigned int FINDDLGH);
__declare_global_int(unsigned int FINDDLGW);
__declare_global_int(unsigned int FINDDLGFW);
__declare_global_int(unsigned int DLGLH);
__declare_global_int(int KEYUP);
__declare_global_int(int KEYDOWN);
__declare_global_int(int KEYRIGHT);
__declare_global_int(int KEYLEFT);
__declare_global_int(int KEYDIAM);
__declare_global_int(int KEYSHIFT);

#define KEYESC 264
#define KEYENTER 13
#define KEYAPPS 265
#define KEYF1 268
#define KEYF2 269
#define KEYF3 270
#define KEYF4 271
#define KEYF8 275
#define KEY2ND 4096
#define KEYALPHA 32768

int _ti89,_ti92plus;

// (16) Setup for this calculator
void CalcSetup() {
	if(TI89) {
		MAXX=160;				// screen sizes
		MAXY=100;
		MAXLINE=12;			// maximum number of lines
		OPENMENULEN=60;	// maximum length of the Open menu
		ABOUTDLGW=153;  // About dialog
		ABOUTDLGH=90;
		FINDDLGW=152;   // Find dialog
		FINDDLGH=55;
		FINDDLGFW=23;   // x*6 = input field width
		DLGLH=7;        // line height in dialogs
	} else {
		MAXX=240;
		MAXY=128;
		MAXLINE=16;
		OPENMENULEN=88;
		ABOUTDLGW=240;
		ABOUTDLGH=110;
		FINDDLGW=234;
		FINDDLGH=65;
		FINDDLGFW=36;
		DLGLH=9;
	}
	KEYUP=KEY_UP;
	KEYRIGHT=KEY_RIGHT;
	KEYDOWN=KEY_DOWN;
	KEYLEFT=KEY_LEFT;
	KEYSHIFT=KEY_SHIFT;
	KEYDIAM=KEY_DIAMOND;
}

#endif
