/*
		View for TI89/TI92+
		Copyright (C) 2000  Matus Horvath (m_horvath@geocities.com)

		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

		-------
		Latest version of this program (in source and binary form) can be found
		at http://www.elf.stuba.sk/~horvathm. The GNU General Public Licence can
		be found at http://www.gnu.org.
*/

#define USE_INITS_PATCH

#include <nostub.h>		// If you wish, you may compile a doors version;
											// but it has no advantages except for size (it is
											// cca 1KB smaller). Simply change this line to
											// #include <doors.h>

#include <graph.h>
#include <kbd.h>
#include <args.h>
#include <alloc.h>
#include <vat.h>
#include <dialogs.h>
#include <menus.h>
#include <printf.h>
#include <string.h>
#include <compat.h>


#define DEBUGx				// to display debugging information, etc
#define FINAL					// to build final release

#define VER_HI "1"		// major version number
#define VER_ME "1"    // minor version number
#define VER_LO "2"		// used for small fixes
#define VER_BLD "43"	// build number (increases from 1)
#define VER_SPC ""		// special version string (like alpha,beta,...)

#include "macros.h"

// 17


struct FILE {
	unsigned char *data;
	unsigned char *text; /* =data+4 */
	int length;
	char *name;
};

struct BUF {
	unsigned int l1[BL1SIZE];
	unsigned int l2[BL2SIZE];
	unsigned int l3[BL3SIZE];
	unsigned int l1dpl;
	unsigned int l2dpl;
	unsigned int maxlp;
};

/* (12) Create open dialog */
int AboutDlg() {
	char text[]="View "BUILDVER"\0"\
							"Copyright (C) 2000 Matus Horvath\0"\
							" \0"\
							"This program is licensed under GNU GPL\0"\
							"version 2 or later (see www.gnu.org).\0"\
							" \0"\
							"Homepage: www.elf.stuba.sk/~horvathm\0"\
							"E-mail: m_horvath@geocities.com\0\0";
	unsigned char *start=text;
	int i=0;
	HANDLE hnAboutDlg;

	if((hnAboutDlg=DialogNewSimple(ABOUTDLGW,ABOUTDLGH))==H_NULL) {
		ERROR("Can't create dialog. (1200)");
		return(-1);
	}

	if(DialogAddTitle(hnAboutDlg,"About View",BT_NONE,BT_OK)==H_NULL) {
		HeapFree(hnAboutDlg);
		ERROR("Can't create dialog. (1201)");
		return(-1);
	}

	while(*start!=0) {
		if(DialogAddText(hnAboutDlg,5,15+i*DLGLH,start)==H_NULL) {
			HeapFree(hnAboutDlg);
			ERROR("Can't create dialog. (1202)");
			return(-1);
		}
		i++;
		start+=strlen(start)+1;
	}

	DialogDo(hnAboutDlg,CENTER,CENTER,NULL,NULL);
	FontSetSys(F_4x6);

	HeapFree(hnAboutDlg);
	return 0;
}

/* Convert string to the special format used by vat.h
	 dest=pointer to a buffer 1 byte longer than src*/
char *strconv(unsigned char *dest,char *src) {
	dest[0]=0;
	strcpy(dest+1,src);
	return dest+strlen(src)+1;
}

/* (01) Create open dialog */
int OpenDlg(struct FILE *file) {
	HANDLE hnMenu,hn;
	SYM_ENTRY *SymPtr;
	SCR_RECT sr;
	int count,i;
	char buffer[20];
	unsigned int retval;
	unsigned char* data;

	/*
	ScrRectFill(&(SCR_RECT){0,9,MAXX,9+MAXLINE*LINEHEIGHT-1},
							ScrRect,A_REVERSE);
	*/

	/* Build popup menu */
	if((hnMenu=PopupNew(NULL,OPENMENULEN))==H_NULL) {
		ERROR("Can't create menu. (0100)");
		return -1;
	}

	if(PopupAddText(hnMenu,-1,"EXIT",1)==H_NULL) {
		ERROR("Can't create menu. (0105)");
		HeapFree(hnMenu);
		return -1;
	}

	count=1;
	SymPtr=SymFindFirst(NULL,0);		/* find all folders */
	while(SymPtr) {
		if(PopupAddText(hnMenu,0,SymPtr->name,++count)==H_NULL) {
			ERROR("Can't create menu. (0101)");
			HeapFree(hnMenu);
			return -1;
		}
		SymPtr=SymFindNext();
	}

	for(i=1;i<count;i++) {					/* find all variables in all folders */
		SymPtr=SymFindFirst(strconv(buffer,PopupText(hnMenu,i+1)),1);
		while(SymPtr) {
			/* check if it is text variable */
			if((hn=SymPtr->handle)==H_NULL) {
				ERROR("Invalid VAT entry - null handle. (0102)");
				return -1;
			}
			if((data=(char *)HeapDeref(hn))==NULL) {
				ERROR("Invalid handle - NULL pointer. (0103)");
				return -1;
			}
			if(data[*(int *)data+1]==224) { /* it is a text variable */
				strcpy(buffer,PopupText(hnMenu,i+1));
				strcat(buffer,"\\");
				strcat(buffer,SymPtr->name);
				if(PopupAddText(hnMenu,i+1,buffer,0)==H_NULL) {
					ERROR("Can't create menu. (0104)");
					HeapFree(hnMenu);
					return -1;
				}
			}
			SymPtr=SymFindNext();
		}
	}

	/* Display menu */
	if((retval=PopupDo(hnMenu,5,14,1))==0) {
		HeapFree(hnMenu);
		FontSetSys(F_4x6);
		return 1;		/* 1 means "do not open anything" */
	}
	FontSetSys(F_4x6);

	if(retval==1) {     /* EXIT */
		HeapFree(hnMenu);
		FontSetSys(F_4x6);
		return -1;
	}

	/* Get pointer to given variable */
	if((SymPtr=SymFindPtr(strconv(buffer,PopupText(hnMenu,retval)),0))==NULL) {
		ERROR("Variable does not exist. (0105)");
		HeapFree(hnMenu);
		return -1;
	}
	HeapFree(hnMenu);

	if((hn=SymPtr->handle)==H_NULL) {
		ERROR("Invalid VAT entry - null handle. (0106)");
		return -1;
	}

	if((file->data=(char *)HeapDeref(hn))==NULL) {
		ERROR("Invalid handle - NULL pointer. (0107)");
		return -1;
	}
	file->text=file->data+4;
	file->length=*(int *)file->data;
	file->name=SymPtr->name;

	/* Write variable name on screen */
	sr.xy.x0=0;sr.xy.y0=0;sr.xy.x1=MAXX;sr.xy.y1=6;
	ScrRectFill(&sr,ScrRect,A_REVERSE);
	DrawStr(0,0,"View "BUILDVER,A_REPLACE);
	DrawStr(MAXX-DrawStrWidth(file->name,F_4x6),0,file->name,A_REPLACE);

	return 0;
}

// (14) Open a variable given as string argument
int OpenArg(struct FILE *file) {
	ESI ap;
	SYM_ENTRY *SymPtr;
	HANDLE hn;
	SCR_RECT sr;
	char *name;

	// Get string argument = variable name
	InitArgPtr(ap);
	if(GetArgType(ap)!=STR_TAG) {
		ERROR("Invalid argument specified. (1400)");
		return -1;
	}

	if((name=GetSymstrArg(ap))==NULL) {
		ERROR("GetSymstrArg returned NULL pointer. (1401)");
		return -1;
	}

	// Get pointer to given variable
	if((SymPtr=SymFindPtr(name,0))==NULL) {
		ERROR("Variable does not exist. (1403)");
		return -1;
	}

	if((hn=SymPtr->handle)==H_NULL) {
		ERROR("Invalid VAT entry - null handle. (1404)");
		return -1;
	}

	if((file->data=(char *)HeapDeref(hn))==NULL) {
		ERROR("Invalid handle - NULL pointer. (1405)");
		return -1;
	}
	file->text=file->data+4;
	file->length=*(int *)file->data;
	file->name=SymPtr->name;

	// Write variable name on screen
	sr.xy.x0=0;sr.xy.y0=0;sr.xy.x1=MAXX;sr.xy.y1=6;
	ScrRectFill(&sr,ScrRect,A_REVERSE);
	DrawStr(0,0,"View "BUILDVER,A_REPLACE);
	DrawStr(MAXX-DrawStrWidth(file->name,F_4x6),0,file->name,A_REPLACE);

	return 0;
}

/* (02) Create structure FILE */
int InitFile(struct FILE **file) {
	if((*file=(struct FILE *)malloc(sizeof(struct FILE)))==NULL) {
		ERROR("Not enough memory. (0200)");
		return -1;
	}
	return 0;
}

/* (03) Destroy structure FILE */
void DoneFile(struct FILE *file) {
	if(file!=NULL) free(file);
}

// (04) Returns the position of the first char on next line
int NextLineCP(unsigned int curCP,struct FILE *file) {
	unsigned int newCP=curCP;
	unsigned int i;
	unsigned int totalwidth=0;

	if(file->text[newCP-1]==13) newCP++; // skip the space

	while(1) {
		switch(file->text[newCP]) {
			case 0:           // end of file
				return newCP;
				break;
			case 13:          // end of line - skip the \13
				return ++newCP;
				break;
			case 9:						// TAB
				totalwidth=(totalwidth/TABSIZE+1)*TABSIZE;
				break;
			default:          // some other character
				if(PROPFONT)
					totalwidth+=FontCharWidth(file->text[newCP]);
				else
					totalwidth+=MAXCHARWIDTH;
				break;
		}
		if(totalwidth>MAXX) {		// exceeded line width
			// newCP is index of last character on this line
			// curCP is index of first character on this line - 1
			if(WORDWRAP) {
/*				DISPS("Start");*/
				for(i=newCP;i>curCP;i--) {
					if(file->text[i]==32) break;
				}
/*				DISPI(i);
				DISPC(file->text[i]);
				DISPS("Stop");*/

				return ((i==curCP)?newCP:i+1);
			} else {
				return newCP;
			}
		}
		newCP++;
	}
}

/* (05) Fills a L3 table starting at L2[l2dpl], returns L2[l2dpl+1] */
unsigned int FillBL3(unsigned int l2dpl,struct BUF *buf,struct FILE *file) {
	unsigned int CP,i;

//	DISPSNW("FillBL3");

	buf->l2dpl=l2dpl;
	CP=(buf->l2)[l2dpl];
	for(i=0;i<BL3SIZE;i++) {
		if((buf->l1dpl*BL2SIZE*BL3SIZE+l2dpl*BL3SIZE+i)<=buf->maxlp) {
			(buf->l3)[i]=CP;
			CP=NextLineCP(CP,file);
		}
	}

//	DISPSNW("               ");

	return CP;
}

/* (06) Fills a L2 table starting at L1[l1dpl], returns L1[l1dpl+1] */
unsigned int FillBL2(unsigned int l1dpl,struct BUF *buf,struct FILE *file) {
	unsigned int CP,i,j;

//	DISPSNW("FillBL2");

	buf->l1dpl=l1dpl;
	CP=(buf->l1)[l1dpl];
	for(i=0;i<BL2SIZE;i++) {
		if((l1dpl*BL2SIZE*BL3SIZE+i*BL3SIZE)<=buf->maxlp) {
			(buf->l2)[i]=CP;

			for(j=0;j<BL3SIZE;j++) {	// this is in fact FillBL3
				if((l1dpl*BL2SIZE*BL3SIZE+i*BL3SIZE+j)<=buf->maxlp) {
					CP=NextLineCP(CP,file);
				}
			}

		}
	}

//	DISPSNW("               ");

	return CP;
}

/* (07) Fills a L1 table starting at 0 */
void FillBL1(struct BUF *buf,struct FILE *file) {
	unsigned int CP,i,j,k;

//	DISPSNW("FillBL1");

	CP=0;
	for(i=0;i<BL1SIZE;i++) {
		if((i*BL2SIZE*BL3SIZE)<=buf->maxlp) {
			(buf->l1)[i]=CP;

			for(j=0;j<BL2SIZE;j++) {			// this is in fact FillBL3
				if((i*BL2SIZE*BL3SIZE+j*BL3SIZE)<=buf->maxlp) {

					for(k=0;k<BL3SIZE;k++) {	// this is in fact FillBL3
						if((i*BL2SIZE*BL3SIZE+j*BL3SIZE+k)<=buf->maxlp) {
							CP=NextLineCP(CP,file);
						}
					}

				}
			}

		}
	}

//	DISPSNW("               ");
}

/* (08) Initialize buffer */
int InitBuf(struct BUF **buf) {
	if((*buf=(struct BUF *)malloc(sizeof(struct BUF)))==NULL) {
		ERROR("Not enough memory. (0800)");
		return -1;
	}
	return 0;
}

/* (13) buf->main[LP] = CP of start of line LP */
int FillBuf(struct BUF *buf,struct FILE *file) {
	unsigned int CP=0,LP=0;

	while(file->text[CP]!=0) {     /* =while(!eof) */
		LP++;
		CP=NextLineCP(CP,file);
	}
	buf->maxlp=LP;

	FillBL1(buf,file);
	FillBL2(0,buf,file);
	FillBL3(0,buf,file);

	return 0;
}

/* (09) Destroys line buffer */
void DoneBuf(struct BUF *buf) {
	if(buf!=NULL) free(buf);
}

/* (10) Converts line number to the first character on line */
int LP2CP(unsigned int LP,struct BUF *buf,struct FILE *file) {
	unsigned int needl1dpl,needl2dpl,needl3dpl,pom;

	if(LP>buf->maxlp) LP=buf->maxlp;

	needl1dpl=LP / (BL2SIZE*BL3SIZE);
	pom = LP % (BL2SIZE*BL3SIZE);
	needl2dpl=pom / BL3SIZE;
	needl3dpl=pom % BL3SIZE;

	if((buf->l1dpl)!=needl1dpl) FillBL2(needl1dpl,buf,file);
	if((buf->l2dpl)!=needl2dpl) FillBL3(needl2dpl,buf,file);

	return buf->l3[needl3dpl];
}

// (11) Displays line LP at position 0,posy, using A_REVERSE if CP is
//			between fstart and fstop (used for found text)
void DispLine(unsigned int LP,int posy,unsigned int fstart,
							unsigned int fstop,struct BUF *buf,struct FILE *file) {
	unsigned int start,stop;	  // beginning and end of line to display
	int width;                        // x positon where next char will be drawn
	unsigned int hlp,i;
	int gap=0;                        // gap before current character (for non-proportional font)

	hlp=LP2CP(LP,buf,file);			// get first and last character to process
	start=hlp + ((file->text[hlp-1]==13)?1:0) + ((hlp==0)?1:0);
	hlp=LP2CP(LP+1,buf,file);
	stop=hlp - ((file->text[hlp-1]==13)?1:0);
	width=0;

	for(i=start;i<stop;i++) {
		hlp=((i>=fstart)&&(i<fstop))?A_REVERSE:A_REPLACE;	// invert found chars
		switch(file->text[i]) {
			case 9:			// TAB
				// Note:TAB can not be chosen in the Find dialog, so there
				// is NO code to display INVERTED TAB!
				width=(width/TABSIZE+1)*TABSIZE;
				break;
			default:		// some other character
				// Non-proportional font: every character is 5 pixel wide.
				// We have to center characters, so that 'i' is displayed as
				// __i__, not i____
				if(!PROPFONT) 
					gap=(MAXCHARWIDTH-FontCharWidth(file->text[i])+1)/2;
				
				DrawChar(width+gap,posy,file->text[i],hlp);
				
				if(PROPFONT) width+=FontCharWidth(file->text[i]);
				else width+=MAXCHARWIDTH;
				break;
		}
	}
}

/* (14) Finds a string in text from position pos and returns its CP */
unsigned int FindString(unsigned int pos,unsigned char *str,struct FILE *file) {
	int length,count,i;

	length=strlen(str);
	if(file->text[pos+length-1]==0) return 0; // pos was more then maximum
	//needed for FindNext - if we have found string at the very end of text before

	count=pos;
	while(file->text[count+length]!=0) {
		if(file->text[count]==str[0]) {
			for(i=0;i<=length;i++) if(file->text[count+i]!=str[i]) break;
			if(i==length) return count;  /* found */
		}
		count++;
	}
	return 0; /* not found */
}

/* (15) Displays the Find dialog, returns 1 if OK, 0 if cancelled, -1 else;
	 buf should be initialized to "", have at least 65 bytes allocated,
	 and will be filled with entered text */
int FindDlg(char *buf) {
	HANDLE hnFindDlg;
	int retval;

	if((hnFindDlg=DialogNewSimple(FINDDLGW,FINDDLGH))==H_NULL) {
		ERROR("Can't create dialog. (1500)");
		return(-1);
	}

	if(DialogAddTitle(hnFindDlg,"Find string",BT_OK,BT_CANCEL)==H_NULL) {
		HeapFree(hnFindDlg);
		ERROR("Can't create dialog. (1501)");
		return(-1);
	}

	if(DialogAddText(hnFindDlg,5,15,"Find first occurance of:")==H_NULL) {
		HeapFree(hnFindDlg);
		ERROR("Can't create dialog. (1502)");
		return(-1);
	}

	if(DialogAddRequest(hnFindDlg,5-(TI89?2:5),15+DLGLH*2,"",0,64,FINDDLGFW)==H_NULL) {
		HeapFree(hnFindDlg);
		ERROR("Can't create dialog. (1503)");
		return(-1);
	}

	retval=DialogDo(hnFindDlg,CENTER,CENTER,buf,NULL);
	FontSetSys(F_4x6);

	HeapFree(hnFindDlg);
	return (retval==13);
}

/* (00) */
void _main(void) {
	LCD_BUFFER bufLCD;

	struct FILE *file=NULL;					// input file
	struct BUF *buf=NULL;						// buffer for line ends
	unsigned int LP;								// first line on screen
	char fbuf[65];									// buffer for FindDlg
	unsigned int fstart,fstop;			// CP of start and end of found text

	SCR_RECT sr;
	int key;    										// key read
	unsigned int i;
	int j;

// ------------------

	// Initialize screen
	LCD_save(bufLCD);
	ClrScr();
	FontSetSys(F_4x6);

	CalcSetup();
	sr.xy.x0=0;sr.xy.y0=9;sr.xy.x1=MAXX;sr.xy.y1=9+MAXLINE*LINEHEIGHT-1;

	DrawStr(0,0,"View "BUILDVER,A_REPLACE);
	DrawLine(0,7,MAXX,7,A_NORMAL);
	DrawLine(0,MAXY-7,MAXX,MAXY-7,A_NORMAL);

	// Initialize structures
	if(InitFile(&file)==-1) EXIT;
	if(InitBuf(&buf)==-1) EXIT;

	// Open file
	if(ArgCount()==0) {
		if(OpenDlg(file)!=0) EXIT;
	} else {
		if(OpenArg(file)!=0) EXIT;
	}
	if(FillBuf(buf,file)==-1) EXIT;

	fbuf[0]=0;	// fbuf=""
	fstart=0; fstop=0;

	ScrRectFill(&sr,ScrRect,A_REVERSE);

	LP=0;
	for(i=LP;i<LP+MAXLINE;i++) DispLine(i,(i-LP)*LINEHEIGHT+9,fstart,fstop,buf,file);

	while(1) {
		key=ngetchx();
		if((key==KEYDOWN)) {
			if(LP+MAXLINE<buf->maxlp) {
				ScrRectScroll(&sr,ScrRect,LINEHEIGHT,A_REVERSE);
				DispLine(LP+MAXLINE,(MAXLINE-1)*LINEHEIGHT+9,fstart,fstop,buf,file);
				LP++;
			}
		} else if((key==KEYUP)) {
			if(LP>0) {
				ScrRectScroll(&sr,ScrRect,-LINEHEIGHT,A_REVERSE);
				LP--;
				DispLine(LP,9,fstart,fstop,buf,file);
			}
		} else if((key==KEYDIAM+KEYRIGHT)||(key==KEYDIAM+KEYDOWN)) {
			if(LP+MAXLINE<buf->maxlp) {
				ScrRectFill(&sr,ScrRect,A_REVERSE);
				LP=buf->maxlp-MAXLINE;
				for(i=LP;i<LP+MAXLINE;i++) DispLine(i,(i-LP)*LINEHEIGHT+9,fstart,fstop,buf,file);
			}
		} else if((key==KEY2ND+KEYDOWN)||(key==KEY2ND+KEYRIGHT)||(key==KEYRIGHT)) {
			if(LP+MAXLINE<buf->maxlp) {
				ScrRectFill(&sr,ScrRect,A_REVERSE);
				LP+=MAXLINE;
				if(LP+MAXLINE>buf->maxlp) LP=buf->maxlp-MAXLINE;
				for(i=LP;i<LP+MAXLINE;i++) DispLine(i,(i-LP)*LINEHEIGHT+9,fstart,fstop,buf,file);
			}
		} else if((key==KEYDIAM+KEYLEFT)||(key==KEYDIAM+KEYUP)) {
			if(LP>0) {
				ScrRectFill(&sr,ScrRect,A_REVERSE);
				LP=0;
				for(i=LP;i<LP+MAXLINE;i++) DispLine(i,(i-LP)*LINEHEIGHT+9,fstart,fstop,buf,file);
			}
		} else if((key==KEY2ND+KEYUP)||(key==KEY2ND+KEYLEFT)||(key==KEYLEFT)) {
			if(LP>0) {
				ScrRectFill(&sr,ScrRect,A_REVERSE);
				if(LP<MAXLINE) LP=0; else LP-=MAXLINE;
				for(i=LP;i<LP+MAXLINE;i++) DispLine(i,(i-LP)*LINEHEIGHT+9,fstart,fstop,buf,file);
			}
		} else if((key==KEYESC)) {
			switch(OpenDlg(file)) {
				case -1:
					EXIT;
				break;
				case 0:
					if(FillBuf(buf,file)==-1) EXIT;

					fbuf[0]=0;	// fbuf=""
					fstart=0; fstop=0;

					ScrRectFill(&sr,ScrRect,A_REVERSE);

					LP=0;
					for(i=LP;i<LP+MAXLINE;i++) DispLine(i,(i-LP)*LINEHEIGHT+9,fstart,fstop,buf,file);
				break;
				case 1:
				break;
			}
		} else if((key==KEYF1)) {
			if(AboutDlg()==-1) EXIT;
		} else if((key==KEYF2)) {
			WORDWRAP=!WORDWRAP;

			if(FillBuf(buf,file)==-1) EXIT;

			ScrRectFill(&sr,ScrRect,A_REVERSE);

			if(LP+MAXLINE>buf->maxlp) {
				if(buf->maxlp<MAXLINE) LP=0;
				else LP=buf->maxlp-MAXLINE;
			}
			for(i=LP;i<LP+MAXLINE;i++) DispLine(i,(i-LP)*LINEHEIGHT+9,fstart,fstop,buf,file);
		} else if((key==KEYF4)) {
			PROPFONT=!PROPFONT;

			if(FillBuf(buf,file)==-1) EXIT;

			ScrRectFill(&sr,ScrRect,A_REVERSE);

			if(LP+MAXLINE>buf->maxlp) {
				if(buf->maxlp<MAXLINE) LP=0;
				else LP=buf->maxlp-MAXLINE;
			}
			for(i=LP;i<LP+MAXLINE;i++) DispLine(i,(i-LP)*LINEHEIGHT+9,fstart,fstop,buf,file);
		} else if((key==KEYF3)||(key==KEYF8)) {
			if(((fstart==0)&&(fstop==0))||(key==KEYF8)) {
				// we don't know what to find or user pressed f8
				fstart=0; // start from beginning of text
				j=FindDlg(fbuf);
			} else {
				fstart++; // start searching from current found text+1
				j=1; // dialog was not displayed, so no error could occur
			}

			switch(j) {
				case 0:
				break;
				case 1:
					fstart=FindString(fstart,fbuf,file);
					if(fstart!=0) { // found sth
						fstop=fstart+strlen(fbuf);
						// find LP of first line after the found text (i)
						for(i=0;i<=buf->maxlp;i++) if(LP2CP(i,buf,file)>fstart) break;
						i--;

						if((i<LP)||(i>=LP+MAXLINE)) {   // line i is not on screen
							LP=i;
							if(LP+MAXLINE>buf->maxlp) LP=buf->maxlp-MAXLINE;
						}

						ScrRectFill(&sr,ScrRect,A_REVERSE);
						for(i=LP;i<LP+MAXLINE;i++) DispLine(i,(i-LP)*LINEHEIGHT+9,fstart,fstop,buf,file);
					} else {
						fstart=0;
						fstop=0;
						ScrRectFill(&sr,ScrRect,A_REVERSE);
						for(i=LP;i<LP+MAXLINE;i++) DispLine(i,(i-LP)*LINEHEIGHT+9,fstart,fstop,buf,file);
					}
				break;
				default:
					EXIT;
				break;
			}
		} else if((key==KEYDIAM+KEYF3)||(key==KEYDIAM+KEYF8)) {
			fstart=0;
			fstop=0;
			ScrRectFill(&sr,ScrRect,A_REVERSE);
			for(i=LP;i<LP+MAXLINE;i++) DispLine(i,(i-LP)*LINEHEIGHT+9,fstart,fstop,buf,file);
		}
	}

	_END_

	DoneBuf(buf);
	DoneFile(file);
	LCD_restore(bufLCD);
}
